#ifndef _FileFormatDefs_H_
#define _FileFormatDefs_H_

/*! 
Structure for the \b Sector data format
*/
typedef struct
{
	short wallptr, wallnum;
	long ceilingz, floorz;
	short ceilingstat, floorstat;
	short ceilingpicnum, ceilingheinum;
	signed char ceilingshade;
	char ceilingpal, ceilingxpanning, ceilingypanning;
	short floorpicnum, floorheinum;
	signed char floorshade;
	char floorpal, floorxpanning, floorypanning;
	char visibility, filler;
	short lotag, hitag, extra;
} sectortype;


/*! 
Structure for the \b Wall data format
*/
typedef struct
{
	long x, y;
	short point2, nextwall, nextsector, cstat;
	short picnum, overpicnum;
	signed char shade;
	char pal, xrepeat, yrepeat, xpanning, ypanning;
	short lotag, hitag, extra;
} walltype;


/*! 
Structure for the \b Sprite data format
*/
typedef struct
{
	long x, y, z;
	short cstat, picnum;
	signed char shade;
	char pal, clipdist, filler;
	unsigned char xrepeat, yrepeat;
	signed char xoffset, yoffset;
	short sectnum, statnum;
	short ang, owner, xvel, yvel, zvel;
	short lotag, hitag, extra;
} spritetype;


/*! 
Structure for storing 3D Positions
*/
typedef struct
{
	int x;
	int y;
	int z;
}Pos;


/*! 
Structure for one \em entire \b Map
*/
typedef struct
{
	/*! Map file version */
	unsigned int nVersion;

	/*! Player start position */
	Pos StartPosition;

	/*! Player start angle */
	unsigned short nAngle;

	/*! Index of the sector in which the player starts */
	unsigned short nStartPointSecNum;

	/*! Total number of sectors in this map */
	unsigned short nNumSectors;

	/*! Sector data for the map */
	sectortype sectors[1024];

	/*! Total number of walls in this map */
	unsigned short nNumWalls;

	/*! Wall data for this map */
	walltype walls[8192];

	/*! Total number of sprites in the map */
	unsigned short nNumSprites;

	/*! Sprite data for this map */
	spritetype sprites[4096];
}MAPSTRUCT;


/*! 
Structure for one tile
*/
typedef struct
{
	/*! X dimension of the \b tile */
	uint16 tilesizex;

	/*! Y dimension of the \b tile */
	uint16 tilesizey;
	
	/*! This stores a few attributes for the \b tile that you can set inside EDITART */
	uint32 picanm;

	/*! Rectangular raw data of the \b tile */
	uint8* data;
}TILESTRUCT;


/*! 
Structure for one \em entire \b ART file
*/
typedef struct
{
public:
	void Clear()
	{
		tilesizex = 0x00;
		tilesizey = 0x00;
		picanm = 0x00;
		tiles = 0x00;
	}

	/*! Art file version */
	unsigned long nVersion;

	/*! Number of tiles in the \b ART file. 
		\b NOTE: don't use this number. If you want to find out how many tile are in
		this file take (nTileEnd - nTileStart).
	*/
	unsigned long nNumTiles;

	/*! Number of the \em first tile in the \b ART file */
	unsigned long nTileStart;

	/*! Number of the \em last tile in the \b ART file */
	unsigned long nTileEnd;

	/*! This is a temp variable */
	uint16 *tilesizex; 

	/*! This is a temp variable */
	uint16 *tilesizey;

	/*! This is a temp variable */
	uint32 *picanm;

	/*! Array of tiles */
	TILESTRUCT *tiles;
}ARTSTRUCT;


/*! Structure for file information */
typedef struct
{
	/*! Filename \b Note: this filename is 8.3 - remember, this excludes the \0 end character */
	char name[12];

	/*! This is the size of the data */
	unsigned long int size;
}FILEHEADERSTRUCT;


/*!  Structure contains both the file information and the raw file data */
typedef struct
{
	/*! Header of information for the file */
	FILEHEADERSTRUCT fileheader;

	/*! The raw file data */
	unsigned char* data;

}FILESTRUCT;


/*! For the un-informed, Ken created the Build Engine. */
static const char* AuthorName = "KenSilverman";


/*! Palette data format */
typedef struct
{
	/*!  
		This 768 byte array is exactly the palette you want.  The format is:
		Red0, Green0, Blue0, Red1, Green1, Blue1, ..., Blue255
		The colors are based on the VGA 262,144 color palette.  The values range from
		0-63, so if you want to convert it to a windows palette you will have to
		multiply each byte by 4.	
	*/
	uint8 palette[768];

	/*!
		The number of shading tables used.  Usually this number is 32,
		but 16 or 64 have also been used.  Each of the 256 colors of the VGA palette
		can take on any of "numpalookups" number of shades.	
	*/
	uint16 numpalookups;	

	/*!  
		The shading table. If numpalookups = 32, then this table is:
		(32 shades) * (256 colors) = 8192 bytes (8K).  The shade tables are often
		made to go from normal brightness (shade #0) down to pitch black (shade #31)
		So the first 256 bytes of the table would be for shade #0, etc...	
	*/
	uint8 (*palookup)[256];

	/*!  
		64K translucent lookup table.  Given any 2 colors of the palette,
		this lookup table gives the best match of the 2 colors when mixed together.
	*/
	uint8 transluc[256][256];

}PALETTESTRUCT;

#endif
